;
; mg.ahk - Mouse gestures with right button.
;
; Jason Hood, 15 August to 9 September, 2004.
;
; Adapted from mousegesture.ahk from:
;   http://lukewarm.s41.xrea.com/AutoHotkey/myscripts/
;
; 30 October, 2006: just toggle the right button instead of suspending.
; 30 August, 2007:  same for disabled windows.
;
; 14 to 17 July, 2009:
;   added gestures to position & size to half or quarter of the screen;
;   added About dialog via tray menu item;
;   only display the tooltip once the mouse has moved; removed its option;
;   borrowed deguix' enabled/disabled icons from his version;
;   read settings via ini file.
;
; v2.00, 28 August to 15 September, 2011:
;   Win+Middle button toggles middle click or left double;
;   display the name of a gesture;
;   use the label to test if the gesture exists;
;   only show the tooltip after a slight pause;
;   recognise diagonal movement (which gestures use instead of the doubles);
;   gestures are more flexible;
;   automatically disable with full-screen applications;
;   replace sensitivity with minimum and maximum;
;   actually draw the gesture, using the tooltip for its name;
;   use PostMessage rather than WinClose;
;   move the actions into a separate file;
;   added an action to add a title to the ini file;
;   added actions to minimize/maximize/restore a window or all windows;
;   added actions to Send or Run something;
;   added actions to use PostMessage;
;   label the actions, not the gestures;
;   press the left or middle button during a gesture to cancel it;
;   toggle the other buttons (when possible) and wheel as well;
;   use Click to work with swapped buttons;
;   only test exclude/disable when the active window is under the cursor;
;   gestures can be configured dynamically, per application;
;   modify some actions to use the active window; use the Win key for cursor;
;   use the Windows key to override window matching;
;   allow translation of program text;
;   allow hotkeys to be assigned to actions;
;   solved the simultaneous buttons issue.
;
; v2.01, 28 April, 2014:
;   new action Topmost to toggle the topmost state of a window;
;   support 1.0 & 1.1 (needed for DllCall).
;
; v2.02, 11 to 18 January, 2019:
;   drop support for 1.0 (but without fully utilising 1.1);
;   automatically exclude borderless windows (client area covers the whole
;    window; replaces full-screen test);
;   add "included" setting to override the above;
;   use "(*FAIL)" instead of ".^" as the dummy regex.
;

#NoEnv
#KeyHistory 0
ListLines Off
SetBatchLines -1
SetMouseDelay -1
SetDefaultMouseSpeed 0
CoordMode Mouse, Screen

if !A_IsCompiled
  Menu Tray, Icon, mg_1.ico

; Read strings from the ini file for translation/customization.
IniRead mg_s1,	   mg.ini, Strings, title_on,  Mouse Gestures
IniRead mg_sAbout, mg.ini, Strings, about,     About

Menu Tray, Tip, %mg_s1%
Menu Tray, Add, %mg_sAbout%, About

goto mg_init


; Hotkeys for script control.
#g::
  mg_state ^= 1
  Hotkey RButton, Toggle
  Hotkey #RButton, Toggle
  Hotkey WheelUp, Toggle
  Hotkey WheelDown, Toggle
  Hotkey LButton, % (mg_state || debounce) ? "On" : "Off"
  Hotkey MButton, % (mg_state || M_is_2L)  ? "On" : "Off"

mg_tray:
  if A_IsCompiled
    Menu Tray, Icon, %A_ScriptName%, % (mg_state) ? 1 : 3
  else
    Menu Tray, Icon, mg_%mg_state%.ico
  Menu Tray, Tip, % mg_s%mg_state%
return


+#g::Reload
^#g::Edit
!#g::ExitApp


#MButton::
  M_is_2L ^= 1
  Tooltip % mg_sM%M_is_2L%
  SetTimer mg_remove, -500
  Hotkey MButton, % (mg_state || M_is_2L) ? "On" : "Off"
return


;------------------------------------------------------------------------------

mg_init:

mg_active := mbL := mbR := mbM := false
mg_state = 1

IniRead mg_s0,	      mg.ini, Strings, title_off, %mg_s1% (disabled)
IniRead mg_sM0,       mg.ini, Strings, middle,	  Middle click
IniRead mg_sM1,       mg.ini, Strings, double,	  Double click
IniRead mg_sUAction,  mg.ini, Strings, uaction,   unknown action: "`%action`%"
IniRead mg_sUStroke,  mg.ini, Strings, ustroke,   unknown stroke: "`%stroke`%"
IniRead mg_sUHotkey,  mg.ini, Strings, uhotkey,   unknown hotkey: "`%key`%"
IniRead mg_sRunfail,  mg.ini, Strings, runfail,   Failed to launch "`%mg_arg`%" (error `%A_LastError`%).
IniRead mg_sConfig,   mg.ini, Strings, config,	  If a window title matches the regular expression:
IniRead mg_sExclude,  mg.ini, Strings, exclude,   Exclude
IniRead mg_sInclude,  mg.ini, Strings, include,   Include
IniRead mg_sDisable,  mg.ini, Strings, disable,   Disable
IniRead mg_sAlready0, mg.ini, Strings, already0,  "`%mg_title`%" already matches disabled.
IniRead mg_sAlready1, mg.ini, Strings, already1,  "`%mg_title`%" already matches included.
IniRead mg_sAlready2, mg.ini, Strings, already2,  "`%mg_title`%" already matches excluded.
IniRead mg_sAlready3, mg.ini, Strings, already3,  "`%mg_title`%" already matches disabled and middle click.
IniRead mg_sAlready4, mg.ini, Strings, already4,  "`%mg_title`%" already matches included and middle click.
IniRead mg_sAlready5, mg.ini, Strings, already5,  "`%mg_title`%" already matches excluded and middle click.
IniRead mg_sAlready6, mg.ini, Strings, already6,  "`%mg_title`%" already matches middle click.
IniRead mg_sOK,       mg.ini, Strings, ok,	  OK
IniRead mg_sCancel,   mg.ini, Strings, cancel,	  Cancel
IniRead mg_sMove,     mg.ini, Strings, move,	  Move "`%mg_atitle`%":
IniRead mg_sLeft,     mg.ini, Strings, left,	  Left:
IniRead mg_sTop,      mg.ini, Strings, top,	  Top:
IniRead mg_sRight,    mg.ini, Strings, right,	  Right:
IniRead mg_sBottom,   mg.ini, Strings, bottom,	  Bottom:
IniRead mg_sHSrc,     mg.ini, Strings, hsrc,	  H. Src:
IniRead mg_sHDst,     mg.ini, Strings, hdst,	  H. Dst:
IniRead mg_sVSrc,     mg.ini, Strings, vsrc,	  V. Src:
IniRead mg_sVDst,     mg.ini, Strings, vdst,	  V. Dst:
IniRead mg_sWidth,    mg.ini, Strings, width,	  Width:
IniRead mg_sHeight,   mg.ini, Strings, height,	  Height:
IniRead mg_sRelative, mg.ini, Strings, relative,  &Relative to "`%mg_ctitle`%"
IniRead mg_sCurrent,  mg.ini, Strings, current,   Current
IniRead mg_sPrevious, mg.ini, Strings, previous,  Previous
IniRead mg_sClear,    mg.ini, Strings, clear,	  Clear
IniRead mg_sRetrieve, mg.ini, Strings, retrieve,  Retrieve
IniRead mg_sDims,     mg.ini, Strings, dims,	  `%dw`% x `%dh`%

; If there's no movement during this period, just right click (milliseconds).
IniRead mg_timeout, mg.ini, Settings, timeout, 500

; A movement requires this many pixels to register as a stroke.
IniRead mg_min, mg.ini, Settings, minimum, 30
mg_min2 := mg_min * mg_min

; Exceeding this length (from the initial click) turns off the gesture.
IniRead mg_max, mg.ini, Settings, maximum, 150
mg_d := mg_max * 2

; Line width & colours of the drawing.
IniRead mg_draw, mg.ini, Settings, draw, 1
IniRead mg_penw, mg.ini, Settings, width, 3
IniRead mg_penic, mg.ini, Settings, color, 0x40FF40
IniRead mg_penoc, mg.ini, Settings, outline, 0x000000

; Window titles to exclude - process right button as normal.
IniRead mg_excluded, mg.ini, Settings, excluded
if !valid_regex(mg_excluded)
  return

; Window titles to include - override automatic exclusion.
IniRead mg_included, mg.ini, Settings, included
if !valid_regex(mg_included)
  return

; Window titles which totally disable the gestures.
IniRead mg_disabled, mg.ini, Settings, disabled
if !valid_regex(mg_disabled)
  return

; Debounce the left button - ignore consecutive clicks within this period (ms).
IniRead debounce, mg.ini, Settings, debounce, 0

; Treat the middle button as a double left-click.
IniRead M_is_2L, mg.ini, Settings, M_is_2L, 1
IniRead mg_mbutton, mg.ini, Settings, mbutton
if !valid_regex(mg_mbutton)
  return

; Read the gestures and their actions.
mg_ini := false
,mg_err := ""
,mg_titles := 0
Loop Read, mg.ini
{
  first := SubStr(A_LoopReadLine, 1, 1)
  if (first = "" || first = ";")
    continue
  if first = [
  {
    mg_ini := A_LoopReadLine = "[Gestures]"
    continue
  }
  if !mg_ini
    continue

  if !RegExMatch(A_LoopReadLine, "^\s*(?P<key>[^=]+?)\s*=\s*(?P<val>.*?)\s*$", mg_)
    continue
  if mg_key = title
  {
    if mg_val =
      mg_atitle =
    else
    {
      if !valid_regex(mg_val)
	return
      mg_atitle := mg_val
      ,++mg_titles
    }
    continue
  }
  if mg_key = class
  {
    if mg_val =
      mg_aclass =
    else
    {
      if !valid_regex(mg_val)
	return
      mg_aclass := mg_val
      ,++mg_titles
    }
    continue
  }
  name := action := arg := ""
  mg_line := A_Index
  Loop Parse, mg_val, CSV
  {
    if A_Index = 1
    {
      name = %A_LoopField%
    }
    else if A_Index = 2
    {
      action = %A_LoopField%
    }
    else
    {
      if A_Index <> 3
	arg .= ","
      arg .= A_LoopField
    }
  }
  if action =
    action := name
  else if name =
    name := action
  mg_action =
  if action is alpha
  {
    mg_action = %action%_Action
    if !IsLabel(mg_action)
      mg_action =
  }
  if mg_action =
  {
    Transform mg_action, Deref, %mg_sUAction%
    mg_err .= "mg.ini:" mg_line ": " mg_action "`n"
    continue
  }
  Loop Parse, mg_key, CSV, %A_Space%%A_Tab%
  {
    gesture := add_gesture(A_LoopField, action)
    if !gesture
      continue
    %gesture%_action := action
    ,%gesture%_name := name
    if arg <>
      %gesture%_arg := arg
  }
}
if mg_err
  MsgBox 0x10, %mg_s1%, %mg_err%


; Create a RECT structure for GetClientRect.
VarSetCapacity(rect, 16)

#include mg_action.ahk


valid_regex(ByRef rx)
{
  global

  if rx =
  {
    ; An empty pattern matches everything, but I want it to match nothing.
    ; Instead of testing separately for an empty string, use a dummy pattern.
    rx = (*FAIL)
    return true
  }

  ; Don't care what it matches, only that it compiles.
  RegExMatch("", rx)
  if ErrorLevel = 0
    return true

  mg_err := ErrorLevel ": """
  RegExMatch(ErrorLevel, "offset (\d+)", mg_arg)
  mg_err .= SubStr(rx, mg_arg1 + 1) """"
  MsgBox 0x10, %mg_s1%, Error in regular expression "%rx%":`n`n%mg_err%.
  return false
}


;------------------------------------------------------------------------------

MButton::
  mb = M
goto MouseButton

LButton::
  ; My left button used to be pretty shocking, so debounce it.
  if (debounce && A_PriorHotKey = "LButton" && A_TimeSincePriorHotkey < debounce)
    return
  mb = L

MouseButton:
  if button_action(mb "B")
    return
  ; I like to use middle button as left double-click.
  if (mb = "M" && M_is_2L)
  {
    MouseGetPos,,, mg_cid
    WinGetTitle mg_ctitle, ahk_id %mg_cid%
    if !RegExMatch(mg_ctitle, mg_mbutton)
    {
      Click L 2
      return
    }
  }

ButtonWait:
  mb%mb% := true
  Click %mb% D
  SetTimer ButtonWaitTimer, 10

ButtonWaitTimer:
  if button_state("L") + button_state("R") + button_state("M") = 0
    SetTimer ButtonWaitTimer, off
return

button_state(b)
{
  global

  if (mb%b% && !GetKeyState(b "Button", "P"))
  {
    Click %b% U
    mb%b% := false
  }
  return mb%b%
}


WheelUp::
  mb = WU
goto Wheel

WheelDown::
  mb = WD

Wheel:
  if !button_action(mb)
    Click %mb%
return


button_action(mb)
{
  global

  if !mg_active
    return false

  if mg_str <>
    mg_cancelled := true
  else
  {
    mg_str := mb
    gosub mg_exec
    mg_str := ""
    ,mg_button := true
  }
  return true
}


Hotkey:
  if mg_gui
    return
  WinGet mg_aid,, A
  WinGetTitle mg_atitle, A
  WinGetClass mg_aclass, A
  mg_winkey := false
  ,mg_cid := mg_aid
  ,mg_ctitle := mg_atitle
  ,mg_control := ""
  ,mg_same := true
  ,mg_str := strip_modifiers(A_ThisHotkey)

mg_exec:
  mg_str := find_gesture()
  if !%mg_str%_action
    return
  Transform mg_action, Deref, `%%mg_str%_action`%_Action
  mg_arg := %mg_str%_arg
goto %mg_action%


#RButton::
  mg_winkey := true
goto Gesture

RButton::
  mg_winkey := false

Gesture:
  ; Don't process whilst one of our GUIs is open.
  if mg_gui
    return
  MouseGetPos px, py, mg_cid, mg_control
  WinGetTitle mg_ctitle, ahk_id %mg_cid%
  WinGet mg_aid,, A
  mg_same := (mg_aid = mg_cid && !mg_winkey)
  if mg_same
  {
    if RegExMatch(mg_ctitle, mg_disabled)
      goto #g
    WinGetPos,,, w, h, ahk_id %mg_cid%
    DllCall("GetClientRect", "ptr", mg_cid, "ptr", &rect)
    if ((w = NumGet(rect, 8, "int") && h = NumGet(rect, 12, "int"))
	|| RegExMatch(mg_ctitle, mg_excluded))
       && !RegExMatch(mg_ctitle, mg_included)
    {
      mb = R
      goto ButtonWait
    }
  }
  WinGetTitle mg_atitle, A
  WinGetClass mg_aclass, A
   mx := mx_ := px2 := px
  ,my := my_ := py2 := py
  ,mg_active := true
  ,mg_button := mg_cancelled := mg_rheld := mg_tip := false
  ,mg_idletime := A_TickCount
  ,mg_str := mg_dir := mg_last_dir := ""
  ,d1 := mg_min
  ,ri := 0
  Loop
  {
    if mg_cancelled
    {
      mg_active := false
      goto mg_remove
    }
    if !GetKeyState("RButton", "P")
    {
      mg_active := false
      gosub mg_remove
      if !mg_button
      {
	if !mg_rheld
	{
	  if mg_str <>
	  {
	    add_ratio(false)
	    goto mg_exec
	  }
	  Click R
	}
	else
	  Click R U
      }
      return
    }
    if !mg_button
    {
      if (A_TickCount - mg_idletime < mg_timeout)
      {
	MouseGetPos dx, dy
	if (dx = mx && dy = my)
	{
	  if (!mg_tip && A_TickCount - mg_idletime >= 250 && add_ratio(false))
	  {
	    mg_tip := find_gesture(), --ri
	    ToolTip % %mg_tip%_name = "" ? mg_tip : %mg_tip%_name
	    mg_idletime := A_TickCount
	  }
	}
	else if (abs(dx - px) >= mg_max || abs(dy - py) >= mg_max)
	{
	  ; Time out on the next loop.
	  mg_idletime := A_TickCount - mg_timeout
	}
	else
	{
	   mx := dx
	  ,my := dy
	  if mg_tip
	  {
	    Tooltip
	    mg_tip := false
	  }
	  mg_idletime := A_TickCount

	  ,dx_ := mx - mx_
	  ,dy_ := my - my_
	  if (abs(dx_) >= 6 || abs(dy_) >= 6)
	  {
	    mg_cur_dir := direction(dx_, dy_, dx_ * dx_ + dy_ * dy_)
	    if mg_cur_dir <> %mg_last_dir%
	    {
	      add_ratio(true)
	      ,mg_last_dir := mg_cur_dir
	      ,px2 := mx_
	      ,py2 := my_
	      ,mg_dir := ""
	    }
	     mx_ := mx
	    ,my_ := my
	  }

	   dx -= px2
	  ,dy -= py2
	  ,L2 := dx * dx + dy * dy
	  if L2 >= %mg_min2%
	  {
	    if mg_dir =
	    {
	      mg_dir := direction(dx, dy, L2)
	      ,mg_str .= mg_dir
	    }
	    if (mg_draw)
	    {
	      gosub mg_create
	      DllCall("LineTo", "ptr", mg_dc, "int", mx - mg_x, "int", my - mg_y)
	      ,DllCall("SelectObject", "ptr", mg_dc, "ptr", mg_peni)
	      ; Redraw the previous two segments to remove the outline.
	      ,DllCall("MoveToEx", "ptr", mg_dc, "int", mx0 - mg_x, int, my0 - mg_y, int, 0)
	      ,DllCall("LineTo", "ptr", mg_dc, "int", mx1 - mg_x, "int", my1 - mg_y)
	      ,DllCall("LineTo", "ptr", mg_dc, "int", mx2 - mg_x, "int", my2 - mg_y)
	      ,DllCall("LineTo", "ptr", mg_dc, "int", mx - mg_x, "int", my - mg_y)
	      ,DllCall("SelectObject", "ptr", mg_dc, "ptr", mg_peno)
	      ,mx0 := mx1, my0 := my1
	      ,mx1 := mx2, my1 := my2
	      ,mx2 := mx,  my2 := my
	    }
	  }
	}
      }
      else if !mg_rheld
      {
	gosub mg_remove
	MouseGetPos px2, py2
	if (px2 <> px || py2 <> py)
	{
	  Click R %px% %py% D
	  MouseMove %px2%, %py2%
	  mg_active := false
	  ,mg_rheld := true
	}
      }
    }
    Sleep 10
  }


; Strip all the non-letters to create hotkey variables.
strip_modifiers(key)
{
  ; Start with k to indicate it is a hotkey, not a gesture.
  s = k
  Loop Parse, key
  {
    if A_LoopField is alnum
      s .= A_LoopField
  }
  return s
}


; Create an array of gestures, without the distances and/or with the titles.
add_gesture(gesture, action)
{
  global
  local g, gi, d, d1, ri, p
	,r1_min, r1_max, r2_min, r2_max, r3_min, r3_max
	,r4_min, r4_max, r5_min, r5_max, r6_min, r6_max
	,r7_min, r7_max, r8_min, r8_max, r9_min, r9_max

  g := SubStr(gesture, 1, 1)
  if g in +,^,!,#,<,>,*
  {
    Hotkey %gesture%, Hotkey, UseErrorLevel
    if ErrorLevel
    {
      key := gesture
      Transform mg_action, Deref, %mg_sUHotkey%
      mg_err .= "mg.ini:" mg_line ": " mg_action "`n"
      return ""
    }
    g := strip_modifiers(gesture)
  }
  else
  {
    if gesture in LB,MB,WU,WD
      g := gesture
    else
    {
      ri := 0
      ,d1 := d := 1
      ,p := "."
      ,g := ""
      Loop Parse, gesture
      {
	if A_LoopField between 1 and 5
	{
	  if p is not digit
	  {
	    p := d := A_LoopField
	    continue
	  }
	}
	g .= A_LoopField
	if p in N,S
	{
	  if A_LoopField not in E,W
	  {
	    stroke := p A_LoopField
	    Transform mg_action, Deref, %mg_sUStroke%
	    mg_err .= "mg.ini:" mg_line ": " mg_action "`n"
	    return ""
	  }
	  p := A_LoopField
	  continue
	}
	if A_LoopField not in U,D,L,R,N,S
	{
	  stroke := A_LoopField
	  Transform mg_action, Deref, %mg_sUStroke%
	  mg_err .= "mg.ini:" mg_line ": " mg_action "`n"
	  return ""
	}
	++ri
	,r%ri%_min := (d - 0.5) / d1
	,r%ri%_max := (d + 0.5) / d1
	,d1 := d
	,d := 1
	,p := A_LoopField
      }
    }
  }
  if (g <> gesture || mg_atitle <> "" || mg_aclass <> "")
  {
    ; It seems if the variable doesn't exist, multi-statement won't create it.
    ++%g%_g0
    gi := %g%_g0
    gi = %g%_g%gi%
    if g <> %gesture%
    {
      if SubStr(g, 1, 1) = "k"
      {
	%gi%_key := gesture
	gesture := g
      }
      else
      {
	%g%_strokes := ri
	Loop %ri%
	{
	   %gi%_r%A_Index%_min := r%A_Index%_min
	  ,%gi%_r%A_Index%_max := r%A_Index%_max
	}
      }
    }
    if mg_atitle <>
    {
      %gi%_title := mg_atitle
      gesture = %gesture%_t%mg_titles%
    }
    if mg_aclass <>
    {
      %gi%_class := mg_aclass
      if mg_atitle =
	gesture = %gesture%_t%mg_titles%
    }
    %gi%_gesture := gesture
  }
  return gesture
}


; Find the gesture, taking into account the distances and title.
find_gesture()
{
  global
  local g, gesture, key, found, custom, strokes

  gesture := mg_str
  if mg_str <>
  {
    strokes := %mg_str%_strokes
    Loop % %mg_str%_g0
    {
      g = %mg_str%_g%A_Index%
      custom := false
      if %g%_title
      {
	custom := true
	if !mg_same || !RegExMatch(mg_atitle, %g%_title)
	  continue
      }
      if %g%_class
      {
	custom := true
	if !mg_same || !RegExMatch(mg_aclass, %g%_class)
	  continue
      }
      found := true
      if %g%_r1_min
      {
	Loop %strokes%
	{
	  if (A_Index = 1 && strokes <> 1)
	    continue
	  if (r%A_Index% < %g%_r%A_Index%_min || r%A_Index% >= %g%_r%A_Index%_max)
	  {
	    found := false
	    break
	  }
	}
      }
      else if (%g%_key <> "" && %g%_key <> A_ThisHotkey)
      {
	found := false
      }
      if found
      {
	gesture := %g%_gesture
	if custom
	  break
      }
    }
  }
  return gesture
}


; Add the ratio of this length to the previous length.	If update is true,
; the previous length becomes this length.
add_ratio(update)
{
  global
  local d, dx, dy

   dx := mx_ - px2
  ,dy := my_ - py2
  ,d := dx * dx + dy * dy

  ; It may be a transitional direction, so check the length.
  if d < %mg_min2%
    return false

  ; Use the normalised axis length for the diagonals.
  if (StrLen(direction(dx, dy, d)) = 2)
    d /= 2
  d := sqrt(d)
  ,++ri
  ,r%ri% := d / d1
  if update
    d1 := d
  return true
}


; Given the delta in the X & Y axes and the square of the length, return the
; direction.
direction(dx, dy, L2)
{
  a := acos(abs(dx) / sqrt(L2)) * 57.29578
  if a < 22.5
    return dx < 0 ? "L" : "R"
  if a > 67.5
    return dy < 0 ? "U" : "D"
  return (dy < 0 ? "N" : "S") (dx < 0 ? "W" : "E")
}


mg_create:
  if !mg_gui
  {
    mg_gui := true
    ,mg_x := px - mg_max
    ,mg_y := py - mg_max
    Gui +LastFound +AlwaysOnTop -Caption +ToolWindow
    Gui Color, EEAA99
    WinSet TransColor, EEAA99
    Gui Show, x%mg_x% y%mg_y% w%mg_d% h%mg_d% NoActivate
    WinGet mg_hwnd
     mg_dc := DllCall("GetDC", "ptr", mg_hwnd)
    ,mg_peni := DllCall("CreatePen", "int", 0, "int", mg_penw, "uint", mg_winkey ? mg_penoc : mg_penic)
    ,mg_peno := DllCall("CreatePen", "int", 0, "int", mg_penw + 2, "uint", mg_winkey ? mg_penic : mg_penoc)
    ,mg_penold := DllCall("SelectObject", "ptr", mg_dc, "ptr", mg_peno)
    ,DllCall("MoveToEx", "ptr", mg_dc, "int", px - mg_x, "int", py - mg_y, "int", 0)
    ,mx2 := mx1 := mx0 := px
    ,my2 := my1 := my0 := py
  }
return


mg_remove:
  Tooltip
  if mg_gui
  {
    mg_gui := false
    ,DllCall("SelectObject", "ptr", mg_dc, "ptr", mg_penold)
    ,DllCall("DeleteObject", "ptr", mg_peni)
    ,DllCall("DeleteObject", "ptr", mg_peno)
    ,DllCall("ReleaseDC", "ptr", mg_dc)
    Gui Destroy
  }
return


About:
  Gui Add, Text, center vTitle, %mg_s1%
  Gui Font, underline
  Gui Add, Text, center cBlue gEmail vEmail, Jason Hood
  Gui Font
  Gui Add, Text, center vVers, Version 2.02 (18 January, 2019)
  Gui Font, underline
  Gui Add, Text, center cBlue gWeb wp, http://mg.adoxa.vze.com/
  Gui Font
  Gui Add, Button, y+20 default wp gGuiClose, %mg_sOK%
  GuiControlGet Vers, Pos
  GuiControl Move, Title, w%VersW%
  GuiControl Move, Email, w%VersW%
  Gui Show,, %mg_sAbout%
  mg_gui := true
return

Email:
  Run mailto:jadoxa@yahoo.com.au?Subject=Mouse`%20Gestures
return

Web:
  Run http://mg.adoxa.vze.com/
return

GuiEscape:
GuiClose:
  Gui Destroy
  mg_gui := false
return
